import 'dart:async';

import 'package:antsomi/catalog.dart';
import 'package:antsomi/inbox_items.dart';
import 'package:antsomi/modify_action.dart';
import 'package:antsomi/screen_observer.dart';
import 'package:antsomi_example/sdk.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';

class NotificationSreen extends StatefulWidget {
  const NotificationSreen({
    Key? key,
    required this.onReadMessage,
    required this.onReceiveNewMessage,
    required this.onTapMessage,
  }) : super(key: key);

  final Function(int)? onReadMessage;
  final Function()? onReceiveNewMessage;
  final Function(InboxItem)? onTapMessage;
  @override
  State<NotificationSreen> createState() => _NotificationSreenState();
}

class _NotificationSreenState extends State<NotificationSreen> {
  List<InboxItem> _resultMessage = [];
  List<Catalog> _labels = [];
  late List<bool> _btnState;
  final List<String> _filterCatalogId = [];

  @override
  void initState() {
    super.initState();
    initAllData();

    WidgetsBinding.instance.addPostFrameCallback((_) {
      AntsomiSDK.instance.onNewMessageReceived(callback: (value) {
        widget.onReceiveNewMessage!();
        setState(() {
          _resultMessage.insert(0, value);
        });
      });
    });
    if (!mounted) {
      return;
    }
  }

  Future<void> initAllData() async {
    
    AntsomiSDK.instance.setPageLimit(limit: 20);

    final catalog = await AntsomiSDK.instance.getAllLabels();
    final resultMessage = await AntsomiSDK.instance.getMessage(page: 1);
 

    setState(() {
      _resultMessage = resultMessage;
      _labels = catalog;
      _btnState = List.generate(_labels.length, (i) {
        return false;
      });
    });
  }

  Future<void> filterData() async {
    final resultMessage = await AntsomiSDK.instance
        .getMessage(catalogId: _filterCatalogId, page: 1);
    setState(() {
      _resultMessage = resultMessage;
    });
  }

  int checkRead({required InboxItem message}) {
    if (message.status == 1) {
      setState(() {
        message.status = 2;
      });

      AntsomiSDK.instance
          .modifyAction(messageId: message.itemId, action: ActionType.read);
      return 1;
    }
    return 0;
  }

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          title: const Text(
            "Notifications",
            style: TextStyle(color: Colors.black),
          ),
          backgroundColor: Colors.white,
          shadowColor: Colors.transparent,
        ),
        body: _listViewBuild(
          size: size,
          labels: _labels,
          filterLabels: _filterCatalogId,
          message: _resultMessage,
          onTapTile: (message) {
            widget.onTapMessage!(message);
            final value = checkRead(message: message);
            widget.onReadMessage!(value);
          },
        ));
  }

  Widget _listViewBuild({
    required Size size,
    required List<Catalog> labels,
    required List<String> filterLabels,
    required List<InboxItem> message,
    required void Function(InboxItem) onTapTile,
  }) {
    return Padding(
      padding: const EdgeInsets.only(top: 0.0),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const SizedBox(height: 12),
          SizedBox(
            height: 38,
            width: double.infinity,
            child: ListView.builder(
              itemBuilder: (context, index) {
                final item = labels[index];
                return FilterButton(
                    isPress: _btnState[index],
                    label: item.catalogName,
                    onPress: () {
                      setState(() {
                        _btnState[index] = !_btnState[index];
                        if (_btnState[index]) {
                          _filterCatalogId.add(labels[index].catalogId);
                        } else {
                          if (_filterCatalogId.isNotEmpty &&
                              _filterCatalogId
                                  .contains(labels[index].catalogId)) {
                            _filterCatalogId.remove(labels[index].catalogId);
                          }
                        }
                      });
                      filterData();
                    });
              },
              itemCount: labels.length,
              scrollDirection: Axis.horizontal,
            ),
          ),
          const SizedBox(height: 12),
          Expanded(
            child: Center(
              child: ListView.builder(
                itemBuilder: (context, index) {
                  final item = message[index];

                  return _customTile(
                      buttonLabel1: item.buttonLabel1,
                      buttonLabel2: item.buttonLabel2,
                      size: size,
                      imgSrc: item.imageUrl,
                      heading: item.heading,
                      content: item.content,
                      templateId: item.templateId,
                      buttonURL1: item.buttonAppUrl1,
                      buttonURL2: item.buttonAppUrl2,
                      ctime: item.dateCreated.toIso8601String(),
                      isRead: item.status == 1 ? false : true,
                      onTap: () {
                        onTapTile(item);
                      });
                },
                itemCount: message.length,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

Widget _customTile(
    {required String imgSrc,
    required String heading,
    required String content,
    required String ctime,
    required bool isRead,
    required Size size,
    required String buttonLabel1,
    required String buttonLabel2,
    required String templateId,
    required String buttonURL1,
    required String buttonURL2,
    void Function()? onTap}) {
  return Padding(
    padding: const EdgeInsets.all(4.0),
    child: SizedBox(
      width: size.width * 0.89,
      child: InkWell(
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.all(8.0),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Image.network(
                imgSrc,
              ),
              Container(
                decoration: BoxDecoration(
                  border: Border.all(width: 0, color: Colors.transparent),
                  borderRadius: const BorderRadius.all(Radius.circular(16.0)),
                  color: const Color.fromARGB(245, 237, 237, 237),
                ),
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(heading,
                          textAlign: TextAlign.start,
                          style: const TextStyle(
                            fontWeight: FontWeight.w600,
                            fontSize: 18,
                          )),
                      const SizedBox(height: 12),
                      Text(
                        content,
                        textAlign: TextAlign.start,
                        style: const TextStyle(
                            fontWeight: FontWeight.w300,
                            fontSize: 14,
                            color: Colors.black),
                        maxLines: 5,
                      ),
                      const SizedBox(height: 8),
                      if (templateId != 'annoucements') ...[
                        buttonWithTemplate(
                            templateId: templateId,
                            buttonLabel1: buttonLabel1,
                            buttonLabel2: buttonLabel2,
                            size: size,
                            buttonURL1: buttonURL1,
                            buttonURL2: buttonURL2)
                      ],
                      Align(
                        alignment: Alignment.bottomRight,
                        widthFactor: 3.0,
                        child: Row(
                          crossAxisAlignment: CrossAxisAlignment.end,
                          mainAxisAlignment: MainAxisAlignment.end,
                          children: [
                            Text(
                              ctime,
                              textAlign: TextAlign.end,
                              style: const TextStyle(
                                fontWeight: FontWeight.w100,
                                fontSize: 10,
                              ),
                            ),
                            const SizedBox(
                              width: 8,
                            ),
                            isRead
                                ? Container()
                                : const Icon(
                                    CupertinoIcons.circle_fill,
                                    color: Colors.blue,
                                    size: 8,
                                  )
                          ],
                        ),
                      ),
                    ],
                  ),
                ),
              )
            ],
          ),
        ),
      ),
    ),
  );
}

Widget buttonWithTemplate({
  required String templateId,
  required Size size,
  String buttonLabel1 = "",
  String buttonLabel2 = "",
  String? buttonURL1,
  String? buttonURL2,
}) {
  switch (templateId) {
    case 'coupon':
      return ElevatedButton(
        style: ElevatedButton.styleFrom(
            backgroundColor: const Color.fromARGB(255, 3, 94, 184),
            shadowColor: Colors.transparent,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(10.0),
            ),
            fixedSize: Size(
              size.width * 0.85,
              32.0,
            )),
        child: Text(
          buttonLabel1,
          textAlign: TextAlign.start,
          style: const TextStyle(
              fontWeight: FontWeight.w400, fontSize: 16, color: Colors.white),
        ),
        onPressed: () => _lauchURL(buttonURL1!),
      );
    case 'order_verified':
      return Row(
        children: [
          ElevatedButton(
            style: ElevatedButton.styleFrom(
              backgroundColor: const Color.fromARGB(255, 3, 94, 184),
              shadowColor: Colors.transparent,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(10.0),
              ),
              fixedSize: Size(
                size.width * 0.43,
                32.0,
              ),
            ),
            child: Text(
              buttonLabel1,
              textAlign: TextAlign.start,
              style: const TextStyle(
                  fontWeight: FontWeight.w400,
                  fontSize: 16,
                  color: Colors.white),
            ),
            onPressed: () => _lauchURL(buttonURL1!),
          ),
          const SizedBox(width: 8),
          Expanded(
            child: ElevatedButton(
              style: ElevatedButton.styleFrom(
                backgroundColor: const Color.fromARGB(255, 3, 94, 184),
                shadowColor: Colors.transparent,
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(10.0),
                ),
              ),
              child: Text(
                buttonLabel2,
                textAlign: TextAlign.start,
                style: const TextStyle(
                    fontWeight: FontWeight.w400,
                    fontSize: 16,
                    color: Colors.white),
              ),
              onPressed: () => _lauchURL(buttonURL2!),
            ),
          )
        ],
      );

    default:
      return Container();
  }
}

Future<void> _lauchURL(String url) async {
  final Uri _url = Uri.parse(url);
  if (!await launchUrl(_url)) {
    throw Exception('Could not lauch $_url');
  }
}

class FilterButton extends StatefulWidget {
  FilterButton(
      {Key? key,
      required this.isPress,
      required this.label,
      required this.onPress})
      : super(key: key);
  final bool isPress;
  String label;
  VoidCallback onPress;

  @override
  State<FilterButton> createState() => _FilterButtonState();
}

class _FilterButtonState extends State<FilterButton> {
  bool? _isPress;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    _isPress = widget.isPress;
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 8.0),
      child: TextButton(
        onPressed: () {
          setState(() {
            _isPress = widget.isPress;
          });
          debugPrint(_isPress.toString());
          widget.onPress();
        },
        style: TextButton.styleFrom(
          backgroundColor:
              _isPress! ? const Color.fromARGB(255, 3, 94, 184) : Colors.white,
          shape: RoundedRectangleBorder(
              side: const BorderSide(
                  color: Color.fromARGB(255, 3, 94, 184),
                  width: 1.5,
                  style: BorderStyle.solid),
              borderRadius: BorderRadius.circular(50)),
        ),
        child: Text(
          widget.label,
          style: TextStyle(
              color: _isPress!
                  ? Colors.white
                  : const Color.fromARGB(255, 3, 94, 184),
              fontWeight: FontWeight.bold),
        ),
      ),
    );
  }
}
